"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.convertScssError = exports.resolveCanonicalize = exports.fileSyntax = exports.getDefaultSassImplementation = void 0;
const path_1 = __importDefault(require("path"));
function getDefaultSassImplementation() {
    let sassImplPkg = 'sass';
    try {
        require.resolve('sass');
    }
    catch (ignoreError) {
        try {
            require.resolve('node-sass');
            sassImplPkg = 'node-sass';
        }
        catch (_ignoreError) {
            try {
                require.resolve('sass-embedded');
                sassImplPkg = 'sass-embedded';
            }
            catch (__ignoreError) {
                sassImplPkg = 'sass';
            }
        }
    }
    return require(sassImplPkg);
}
exports.getDefaultSassImplementation = getDefaultSassImplementation;
function fileSyntax(filename) {
    if (filename.endsWith('.scss')) {
        return 'scss';
    }
    else {
        return 'indented';
    }
}
exports.fileSyntax = fileSyntax;
function resolveCanonicalize(importer, file, alias) {
    const importerExt = path_1.default.extname(importer);
    const fileExt = path_1.default.extname(file);
    if (!fileExt) {
        file += importerExt;
    }
    // absolute path
    if (path_1.default.isAbsolute(file)) {
        return file;
    }
    const scope = file.split('/')[0];
    // alias
    if (alias[scope]) {
        const baseFile = file.slice(scope.length + 1);
        return path_1.default.resolve(alias[scope], baseFile);
    }
    return path_1.default.resolve(path_1.default.dirname(importer), file);
}
exports.resolveCanonicalize = resolveCanonicalize;
function convertScssError(error, filePath) {
    const [message, _, lineText, ...rest] = error.message.split('\n');
    const stack = rest[rest.length - 1];
    const lineColumn = stack.match(/(\d+):(\d+)/);
    return {
        text: message,
        location: {
            file: filePath,
            line: Number(lineColumn[1]),
            column: Number(lineColumn[2]),
            lineText: lineText.trim().replace(/\d+\s│/, ''),
        },
    };
}
exports.convertScssError = convertScssError;
