"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.transformSass = void 0;
const sass_utils_1 = require("./sass-utils");
const promises_1 = require("fs/promises");
let sassEngine;
const path_1 = __importDefault(require("path"));
const url_1 = require("url");
const fs = __importStar(require("fs"));
const transformSass = (filePath, options) => __awaiter(void 0, void 0, void 0, function* () {
    const { sourcemap } = options;
    if (!sassEngine) {
        try {
            sassEngine = (0, sass_utils_1.getDefaultSassImplementation)();
        }
        catch (e) {
            console.error(e);
            process.exit(1);
        }
    }
    const syntax = (0, sass_utils_1.fileSyntax)(filePath);
    const basedir = path_1.default.dirname(filePath);
    const contents = yield (0, promises_1.readFile)(filePath, 'utf-8');
    const warnings = [];
    const alias = options.alias || {};
    const result = yield sassEngine.compileStringAsync(contents, {
        sourceMapIncludeSources: true,
        sourceMap: sourcemap,
        syntax,
        alertColor: false,
        logger: {
            warn(message, options) {
                var _a, _b;
                if (!options.span) {
                    warnings.push({ text: `sass warning: ${message}` });
                }
                else {
                    const filename = (_b = (_a = options.span.url) === null || _a === void 0 ? void 0 : _a.pathname) !== null && _b !== void 0 ? _b : path_1.default;
                    const esbuildMsg = {
                        text: message,
                        location: {
                            file: filename,
                            line: options.span.start.line,
                            column: options.span.start.column,
                            lineText: options.span.text,
                        },
                        detail: {
                            deprecation: options.deprecation,
                            stack: options.stack,
                        },
                    };
                    warnings.push(esbuildMsg);
                }
            },
        },
        importer: {
            load(canonicalUrl) {
                const pathname = (0, url_1.fileURLToPath)(canonicalUrl);
                let contents = fs.readFileSync(pathname, 'utf8');
                return {
                    contents,
                    syntax: (0, sass_utils_1.fileSyntax)(pathname),
                    sourceMapUrl: sourcemap ? canonicalUrl : undefined,
                };
            },
            canonicalize(url) {
                const urlPath = (0, sass_utils_1.resolveCanonicalize)(filePath, url, alias);
                if (urlPath) {
                    return (0, url_1.pathToFileURL)(urlPath);
                }
                return null;
            },
        },
    });
    const sourceMap = result.sourceMap;
    if (sourceMap) {
        sourceMap.sourceRoot = basedir;
        sourceMap.sources = sourceMap.sources.map((source) => {
            return path_1.default.relative(basedir, source.startsWith('data:') ? filePath : (0, url_1.fileURLToPath)(source));
        });
    }
    const imports = result.loadedUrls.map((url) => (0, url_1.fileURLToPath)(url));
    return {
        css: result.css,
        map: result.sourceMap ? JSON.stringify(result.sourceMap) : '',
        imports,
        warnings,
    };
});
exports.transformSass = transformSass;
