"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.generateTargets = exports.resolvePath = exports.parsePath = exports.cssExportsToJs = exports.codeWithSourceMap = void 0;
const lightningcss_1 = require("lightningcss");
const path_1 = __importDefault(require("path"));
const browserslist_1 = __importDefault(require("browserslist"));
const camel_case_1 = require("camel-case");
const codeWithSourceMap = (code, map) => {
    return code + '/*# sourceMappingURL=data:application/json;base64,' + Buffer.from(map).toString('base64') + ' */';
};
exports.codeWithSourceMap = codeWithSourceMap;
const cssExportsToJs = (exports, entryFile) => {
    const keys = Object.keys(exports).sort();
    const values = keys.map((key) => exports[key]);
    let exportCode = ``;
    keys.map((key, index) => {
        exportCode += `const ${(0, camel_case_1.camelCase)(key)} = "${values[index].name}";\n`;
    });
    exportCode += `export {${keys
        .map((key) => {
        return `${(0, camel_case_1.camelCase)(key)} as '${key}'`;
    })
        .join(',')}};\n`;
    exportCode += `export default {${keys
        .map((key) => {
        return `'${key}': ${(0, camel_case_1.camelCase)(key)}`;
    })
        .join(',')}};\n`;
    if (entryFile) {
        return `import '${entryFile}';\n${exportCode}`;
    }
    return exportCode;
};
exports.cssExportsToJs = cssExportsToJs;
/**
 * 从路径中解析文件路径和 query
 */
const parsePath = (path) => {
    const queryIndex = path.indexOf('?');
    if (queryIndex === -1) {
        return { path: path, query: '' };
    }
    else {
        return {
            path: path.slice(0, queryIndex),
            query: path.slice(queryIndex + 1),
        };
    }
};
exports.parsePath = parsePath;
function resolvePath(args, build) {
    return __awaiter(this, void 0, void 0, function* () {
        const { path, query } = (0, exports.parsePath)(args.path);
        let absolutePath = path;
        if (!path_1.default.isAbsolute(absolutePath)) {
            const result = yield build.resolve(absolutePath, {
                resolveDir: args.resolveDir,
                importer: args.importer,
                kind: args.kind,
            });
            absolutePath = result.path;
        }
        return { path: absolutePath, query };
    });
}
exports.resolvePath = resolvePath;
const generateTargets = (queries) => {
    return (0, lightningcss_1.browserslistToTargets)((0, browserslist_1.default)(queries));
};
exports.generateTargets = generateTargets;
